#!/bin/sh
#set -xv
UPGRADE_VER="0.0.1"
BIN_PATH="/home/root"
CARD_PATH=${MOUNTR}
UPGRADE_MD5="upgrade.md5"
UPGRADE_SH="Upgrade.sh"
KERN_NAME="uImage.out"
SPL_NAME="boot.bin"
BOOT_NAME="u-boot.img"
FPGA_NAME="fpga.img"
RTFS_NAME="sp4v2_rootfs"
NWRITE_NAME="nwrite"
NAND_WR=${CARD_PATH}/${NWRITE_NAME}
WARTOOL="checkWar"
PROD_NAME="productVersion.txt"
SH_LOG=/tmp/sh.log
LOG_PATH=/mnt/card

EMMCDEV=/dev/mmcblk0
MNT_PATH=/mnt/cf

# PMU
PMU_FW_NAME=SP8.BIN
PMU_LOADER_NAME=pmu_loader
PMU_DEB_LOG=pmu_deb.log

displ() {
 ${PRINTTOOL} --x=20 --y=4  --text="Upgrading" --clear
 ${PRINTTOOL} --x=20 --y=24 --text="Firmware"
 ${PRINTTOOL} --x=20 --y=44 --text="${1}"
 echo "Upgrading ${1}" | tee -a ${LOG}
}

do_reboot() {
#  if [ -f ${CARD_PATH}/upl_start.log ]; then
    echo "Removing temporary files from ${CARD_PATH}" | tee -a ${LOG}
    rm -f ${CARD_PATH}/${RTFS_NAME}* ${CARD_PATH}/${KERN_NAME}* ${CARD_PATH}/${FPGA_NAME}* ${CARD_PATH}/${BOOT_NAME}* ${CARD_PATH}/upl_start.log
    rm -f ${CARD_PATH}/${SPL_NAME}
    rm -f ${CARD_PATH}/${UPGRADE_MD5} ${CARD_PATH}/${NWRITE_NAME} ${CARD_PATH}/${PROD_NAME}
    rm -f ${CARD_PATH}/*.ta_
    rm -f ${CARD_PATH}/${UPGRADE_SH}
    rm -f ${CARD_PATH}/${PMU_FW_NAME} 
    rm -f ${CARD_PATH}/${PMU_LOADER_NAME}
    rm -f ${CARD_PATH}/digest.sign
#  fi
  echo "SMOD 0" > /proc/sp4
  if [ "$1" != "0" ]; then
    echo "UERR 1" > /proc/sp4
  fi
  echo "Message:${2}" | tee -a ${LOG}
  displ ${2}
  # !The next should be the lats message in uploading.log!
  echo "Exit code is ${1}" | tee -a ${LOG}
  if [ -f ${LOG_PATH}/*.ta_ ]; then
    rm -f ${LOG_PATH}/*.ta_   
  fi
  cp ${LOG} ${LOG_PATH}
  
  reboot
  sleep 300
}

do_mount_part() {
  echo "mounting ${EMMCDEV}p$1 to ${MNT_PATH}" | tee -a ${LOG}
  umount ${MNT_PATH} > /dev/null
  # mkfs.vfat ${EMMCDEV}p$1
  fsck.fat -p -w ${EMMCDEV}p$1
  mount -t vfat ${EMMCDEV}p$1 ${MNT_PATH}
  if [ $? -ne 0 ]; then
      echo "mount ${EMMCDEV}p$1 failure. starting mkfs.vfat -v ${EMMCDEV}p$1" | tee -a ${LOG}
      mkfs.vfat -v ${EMMCDEV}p$1
      mount -t vfat ${EMMCDEV}p$1 ${MNT_PATH}
      if [ $? -ne 0 ]; then
        do_reboot -1 "Error 2.$1"
      fi
  fi
  echo "mount ${EMMCDEV}p$1 to ${MNT_PATH} Ok" | tee -a ${LOG}
  ls -l ${MNT_PATH} | tee -a ${LOG}
}

do_file_check() {
 echo "[$(cat /proc/uptime | cut -d ' ' -f1)] Check file ${1}" | tee -a ${LOG}
 hash1=`grep ${1} ${UPGRADE_MD5} | awk '{split($0,x," ");printf("%s",x[1])}'`
 echo "Orig hash $hash1" | tee -a ${LOG}
 hash2=$(md5sum ${1} | awk '{split($0,x," ");printf("%s",x[1])}')
 echo "File hash $hash2" | tee -a ${LOG}
 if [ "${hash1}" != "${hash2}" ]; then
  echo "${1} read failure" | tee -a ${LOG}
  echo "UERR 1" > /proc/sp4
  do_reboot -1 "Error 3"
 fi
}

do_sys_check() {
 chk_name=${CARD_PATH}/${1}
 chk_len=$(ls -l $chk_name | awk '{split($0,x," ");printf("%s",x[5])}')
 dd_len=$(($chk_len/512+1))
 if [ ! -s ${MNT_PATH}/$1 ]; then
   echo "File not found ${MNT_PATH}/$1. Update needed" | tee -a ${LOG}
   return 1
 fi
 chk_hash=`md5sum ${MNT_PATH}/$1 | awk '{split($0,x," ");printf("%s",x[1])}'`
 new_hash=`grep ${1} ${CARD_PATH}/${UPGRADE_MD5} | awk '{split($0,x," ");printf("%s",x[1])}'`
 echo "[$(cat /proc/uptime | cut -d ' ' -f1)] chk_name $chk_name chk_len $chk_len dd_len $dd_len skip $2" | tee -a ${LOG}
 echo "mnt_hash  $chk_hash" | tee -a ${LOG}
 echo "new_hash $new_hash" | tee -a ${LOG}
 if [ "${chk_hash}" != "${new_hash}" ]; then
  echo "[$(cat /proc/uptime | cut -d ' ' -f1)] ${1} md5sum differ" | tee -a ${LOG}
  return 1
 else 
  echo "[$(cat /proc/uptime | cut -d ' ' -f1)] ${1} md5sum Ok" | tee -a ${LOG}
  return 0
 fi
}

do_sys_write() {
 # Img.Name, Offs, Dev.Name
 /usr/local/bin/wd_ping -1 > /dev/null
 do_sys_check $1 $2
 if [ $? -ne 0 ]; then
  echo "[$(cat /proc/uptime | cut -d ' ' -f1)] wr img:${CARD_PATH}/$1 ${MNT_PATH}/$1" | tee -a ${LOG}
  cp ${CARD_PATH}/$1 ${MNT_PATH}/$1
  ret=$?
  sync
  if [ $ret -ne 0 ]; then
    echo "[$(cat /proc/uptime | cut -d ' ' -f1)] ${1} record failure" | tee -a ${LOG}
    return 1
  else
    echo "[$(cat /proc/uptime | cut -d ' ' -f1)] img:${CARD_PATH}/$1 ${MNT_PATH}/$1 write Ok" | tee -a ${LOG}
    return 0
  fi
 else
  echo "[$(cat /proc/uptime | cut -d ' ' -f1)] img:${CARD_PATH}/$1 ${MNT_PATH}/$1 is the same. Write skipped" | tee -a ${LOG}
  return 0
 fi
}

if [ "$fl_resc" == "" ]; then
  fl_resc=0
fi
cd ${CARD_PATH}
rm -f ${LOG}
echo "[$(cat /proc/uptime | cut -d ' ' -f1)] Starting upgrade script ${UPGRADE_VER} at " $(date) "smod=$smod fl_resc=$fl_resc" | tee -a ${LOG}
echo "---${SH_LOG} begin" | tee -a ${LOG}
cat ${SH_LOG} | tee -a ${LOG}
echo "---${SH_LOG} end" | tee -a ${LOG}
ps | grep wd_ping | awk '{split($0,x," ");printf("%s ",x[1])}' | xargs kill
sleep 1
/usr/local/bin/wd_ping -60 -p &
ret=$?
echo "wd_ping ret $ret" | tee -a ${LOG}
if [ $ret -ne 0 ]; then
  /usr/local/bin/wd_ping -60 -p &
fi
uname -a | tee -a ${LOG}
mount | tee -a ${LOG}
ps | grep wd_ping | tee -a ${LOG}
df | tee -a ${LOG}
if [ -f ${BIN_PATH}/${PROD_NAME} ]; then
   echo "Previous product version " $(cat ${BIN_PATH}/${PROD_NAME}) | tee -a ${LOG}
fi
cat /proc/sp4 | tee -a ${LOG}

# Check Warr
pwd | tee -a ${LOG}
tar -x${TAR_OPT} ${TAR_FILE} ${PROD_NAME}
ls -l * | tee -a ${LOG}

if [ $fl_resc -ne 1 ]; then
 WAR_FILE=${BIN_PATH}/${WARTOOL}
 if [ -f ${WAR_FILE} ]; then
 echo "Skip checking Warr at " $(date) | tee -a ${LOG}
 #${WAR_FILE} ./${PROD_NAME}
 #if [ $? -eq 0 ]; then
 # echo "Error to upgrade. Warranty period exceeds !!!" | tee -a ${LOG}
 # echo "Ending upgrade script at " $(date) | tee -a ${LOG}
 # echo "UERR 1" > /proc/sp4
 # do_reboot -1 "Not Allowed"
 #fi
fi
else
 echo "Rescue mode. Skip Warr"
fi

fl_kupl=0
flist="`tar -t$TAR_OPT $TAR_FILE`"
echo "flist:$flist" | tee -a ${LOG}
fl_rfs=`echo $flist | grep -c "$RTFS_NAME"`
fl_kern=`echo $flist | grep -c "$KERN_NAME"`
fl_boot=`echo $flist | grep -c "$BOOT_NAME"`
fl_spl=`echo $flist | grep -c "$SPL_NAME"`
echo "fl_rfs=$fl_rfs, fl_kern=$fl_kern, fl_boot=$fl_boot, fl_spl=$fl_spl" | tee -a ${LOG}
if [[ "${fl_rfs}" -ne "0" && "${fl_kern}" -ne "0" && "${fl_boot}" -ne "0" ]]; then
     fl_kupl=1
     tar -x$TAR_OPT $TAR_FILE $UPGRADE_MD5
     if [ ! -f ${UPGRADE_MD5} ]; then
        echo "Can not find $UPGRADE_MD5. Upgrade cancelled" | tee -a ${LOG}
        do_reboot -1 "Error 1.1"
     fi
     tar -x$TAR_OPT $TAR_FILE $NWRITE_NAME
     if [ ! -f ${NWRITE_NAME} ]; then
        echo "Can not find $NWRITE_NAME. Upgrade cancelled" | tee -a ${LOG}
        do_reboot -1 "Error 1.2"
     fi
     do_file_check ${NWRITE_NAME}
     echo "Upgrade BSP" > ${CARD_PATH}/upl_start.log
     if [ ! -f ${CARD_PATH}/upl_start.log ]; then
         echo "Readonly" | tee -a ${LOG}
         do_reboot -1 "Error 1.3"
     fi
fi

displ "Step 1/5"

mntname=$(echo ${CARD_PATH} | awk '{split($0,x,"/");printf("%s",x[3])}')
freelen=`df | grep ${mntname} | awk '{split($0,x," ");printf("%s",x[4])}'` 
echo "[$(cat /proc/uptime | cut -d ' ' -f1)] ${mntname} free space ${freelen}" | tee -a ${LOG}
if [ $((freelen/1024)) -lt 60 ]; then 
     echo "Low free space on ${CARD_PATH}, ${mntname} free space ${freelen}" | tee -a ${LOG}
     do_reboot -1 "Low free space. Update skipped. Exiting"
fi

#RFS
if [ ! "$fl_rfs" -eq "0" ]; then
     echo "[$(cat /proc/uptime | cut -d ' ' -f1)] unpack $RTFS_NAME to ${CARD_PATH}" | tee -a ${LOG}
     tar -x$TAR_OPT $TAR_FILE $RTFS_NAME
     echo "Unpack $RTFS_NAME ret $?" | tee -a ${LOG}
     do_file_check ${RTFS_NAME}
fi
# KERN
if [ ! "$fl_kern" -eq "0" ]; then
     echo "[$(cat /proc/uptime | cut -d ' ' -f1)] unpack $KERN_NAME to ${CARD_PATH}" | tee -a ${LOG}
     tar -x$TAR_OPT $TAR_FILE $KERN_NAME
     echo "Unpack $KERN_NAME ret $?" | tee -a ${LOG}
     do_file_check ${KERN_NAME}
fi
# FPGA
if [ ! "$fl_kern" -eq "0" ]; then
     echo "[$(cat /proc/uptime | cut -d ' ' -f1)] unpack $FPGA_NAME to ${CARD_PATH}" | tee -a ${LOG}
     tar -x$TAR_OPT $TAR_FILE $FPGA_NAME
     echo "Unpack $FPGA_NAME ret $?" | tee -a ${LOG}
     do_file_check ${FPGA_NAME}
fi
# BOOT
if [ ! "$fl_boot" -eq "0" ]; then
     echo "[$(cat /proc/uptime | cut -d ' ' -f1)] unpack $BOOT_NAME to ${CARD_PATH}" | tee -a ${LOG}
     tar -x$TAR_OPT $TAR_FILE $BOOT_NAME | tar -xj
     echo "Unpack $BOOT_NAME ret $?" | tee -a ${LOG}
     do_file_check ${BOOT_NAME}
fi

if [ "${fl_kupl}" -ne "1" ]; then
  do_reboot -1 "Update skipped. Exiting"
fi

displ "Step 2/5"

echo "${CARD_PATH} content" | tee -a ${LOG}
ls -l . | tee -a ${LOG}

# Main
echo "Write main" | tee -a ${LOG}
do_mount_part 1
do_sys_write ${FPGA_NAME} 1
echo "ret $?" | tee -a ${LOG}
do_sys_write ${BOOT_NAME} 1
echo "ret $?" | tee -a ${LOG}
do_sys_write ${KERN_NAME} 1
echo "ret $?" | tee -a ${LOG}
do_sys_write ${RTFS_NAME} 1
echo "ret $?" | tee -a ${LOG}

displ "Step 3/5"

# Second
echo "Write second" | tee -a ${LOG}
do_mount_part 2
do_sys_write ${FPGA_NAME} 2
echo "ret $ret" | tee -a ${LOG}
do_sys_write ${BOOT_NAME} 2
echo "ret $ret" | tee -a ${LOG}
do_sys_write ${KERN_NAME} 2
echo "ret $ret" | tee -a ${LOG}
do_sys_write ${RTFS_NAME} 2
echo "ret $ret" | tee -a ${LOG}

if [ "$fl_kupl" -eq "1" ]; then
     echo "UCMD KUPL" > /proc/sp4
     # The next line fills environment with 0xFF
     # tr "\000" "\377" < /dev/zero | dd count=1024 of=/dev/mtdblock1
     # echo "Boot env. reset at /dev/mtd2" | tee -a ${LOG}
     # flash_erase dev_name start_offset block_cnt

  # SPL-BOOT
  if [ "$fl_spl" -ne "0" ]; then
    echo "[$(cat /proc/uptime | cut -d ' ' -f1)] unpack ${SPL_NAME} to ${CARD_PATH}" | tee -a ${LOG}
    tar -x$TAR_OPT $TAR_FILE ${SPL_NAME} | tar -xj
    echo "Unpack ${SPL_NAME} ret $?" | tee -a ${LOG}
    if [ -s ${SPL_NAME} ]; then
      do_file_check ${SPL_NAME}
      echo "Update SPL start" | tee -a ${LOG}
      /usr/local/bin/update_spl.sh 1.8V ${CARD_PATH}/${SPL_NAME}
      echo "Update SPL done, ret $?" | tee -a ${LOG}
    else
      echo "${SPL_NAME} not found in tar. Update SPL skipped" | tee -a ${LOG}
    fi
  else
    echo "${SPL_NAME} not found in list. Update SPL skipped" | tee -a ${LOG}
  fi
fi

# PMU upgrade
if [ `grep -c ${PMU_FW_NAME} ${UPGRADE_MD5}` -eq 1 ]; then

  displ "Step 4/5"

  cd ${CARD_PATH}
  echo "[$(cat /proc/uptime | cut -d ' ' -f1)] unpack PMU files to ${CARD_PATH}" | tee -a ${LOG}
  tar -x$TAR_OPT $TAR_FILE ${PMU_FW_NAME} ${PMU_LOADER_NAME}
  echo "Unpack ret $?" | tee -a ${LOG}

  do_file_check ${PMU_FW_NAME}
  do_file_check ${PMU_LOADER_NAME}

  echo "[$(cat /proc/uptime | cut -d ' ' -f1)] Upgrading PMU at " $(date) | tee -a  ${LOG}

  /usr/local/bin/pmu_cmds -v
  if [ $? -eq 0 ]; then
   pmu_ver1=`/usr/local/bin/pmu_cmds -v | awk '{split($0,x,"."); printf("%s",substr(x[1],length(x[1])));}' | hexdump -n 1 -v -e '"%02x"'`
   pmu_ver2=`/usr/local/bin/pmu_cmds -v | awk '{split($0,x,"."); printf("%s",x[2]);}' | hexdump -n 1 -v -e '"%02x"'`
  else
   pmu_ver1="??";
   pmu_ver2="??";
  fi

   f_ver1=`tail -c 8 ${PMU_FW_NAME} | hexdump -v -n 1 -e '"%02x"'`
   f_ver2=`tail -c 7 ${PMU_FW_NAME} | hexdump -v -n 1 -e '"%02x"'`

   echo "pmu_ver:${pmu_ver1}${pmu_ver2},f_ver:${f_ver1}${f_ver2}" | tee -a  ${LOG}
   if [ "${pmu_ver1}${pmu_ver2}" != "${f_ver1}${f_ver2}" ]; then
     echo "starting:./${PMU_LOADER_NAME} -f ${PMU_FW_NAME}" | tee -a ${LOG}
     ./${PMU_LOADER_NAME} -f ${PMU_FW_NAME} -p /dev/spidev0.0 -i -r 964
     if [ ! $? -eq 0 ]; then
       echo "Error to upgrade PMU !!!" | tee -a  ${LOG}
       cat ${PMU_DEB_LOG} | tee -a  ${LOG}
       echo "Ending upgrade script at " $(date) | tee -a ${LOG}
       do_reboot -1 "Error 5.1"
     fi
   else
       echo "PMU upgrade skip." | tee -a  ${LOG}
   fi
   rm -rf ${PMU_DEB_LOG}
fi

displ "Step 5/5"

echo "[$(cat /proc/uptime | cut -d ' ' -f1)] Removing prev.data" | tee -a ${LOG}
rm -rf /mnt/pvt/reg* /mnt/pvt/antenna/
cd /mnt/bin
rm -f CHAN PVT SL sp4*.sh bt_iap pvt_storage.txt productVersion.txt *PMU.bin
rm -rf www
sync
echo "[$(cat /proc/uptime | cut -d ' ' -f1)] Removing prev.data complete" | tee -a ${LOG}

do_reboot 0 "Complete"
